/*
Requires:
<script src="/assets/js/jquery.min.js"></script>
<script src="/assets/js/js.cookie.min.js"></script>
<script src="/assets/js/uuid.js"></script>
<script>
    var pageKey = 'ld-{{ page.i18n-link }}-{{ page.lang }}'
    pingPageLoad(pageKey)
</script>
 */

// boolean
var fabVisible = false

// User
var loggedUserData = undefined

var fabInited = false

/*
Can be called multiple times, and every time is removes old fab

extraMenuLinksFunction: (user: User)=>List<{text: string, link: string, icon: string}>
onSignedEvent: (user: User)=>void
onSignedOutEvent: ()=>void
 */
function initFab(name, newsletter_name, logo_url, extraMenuLinksFunction, onSignedEvent, onSignedOutEvent) {
    if(fabInited) return
    fabInited = true

    var baseUrl = 'https://api.kt.academy/api'
    var userEndpoint = baseUrl + '/user'
    var feedbackEndpoint = baseUrl + '/feedback'

    // *** Network ***

    // Registeres user and returns user data
    function registerUser(googleUser, onRegistered) {
        let basicProfile = googleUser.getBasicProfile();
        let searchParams = new URLSearchParams(window.location.search)
        $.ajax({
            url: userEndpoint + "/google",
            type: 'post',
            data: JSON.stringify({
                email: basicProfile.getEmail(),
                name: basicProfile.getGivenName(),
                surname: basicProfile.getFamilyName(),
                imageUrl: basicProfile.getImageUrl(),
                googleToken: googleUser.getAuthResponse().id_token,
                pageKey: typeof pageKey !== 'undefined' ? pageKey : null,
                userId: searchParams.has("userId") ? searchParams.get("userId") : null,
                workshopGroupAccessKey: searchParams.has("workshopGroupAccessKey") ? searchParams.get("workshopGroupAccessKey") : null,
            }),
            headers: {
                userUuid: userUuid,
                "Content-Type": "application/json"
            },
            dataType: 'json',
            success: function (userData) {
                onRegistered(userData)
            },
            error: function (xhr, ajaxOptions, thrownError) {
                console.log(xhr.status);
                console.log(thrownError);
            }
        });
    }

    // Returns user data if signed in, or null otherwise
    function getUserData(onLoaded) {
        $.ajax({
            url: userEndpoint + "/me",
            type: 'get',
            headers: {
                userUuid: userUuid,
                "Content-Type": "application/json"
            },
            dataType: 'json',
            success: function (userData) {
                onLoaded(userData)
            },
            error: function (xhr, ajaxOptions, thrownError) {
                console.log(xhr.status);
                console.log(thrownError);
            }
        });
    }

    function requestDeleteUserAndSignOutGoogle(onSignedOut, url) {
        $.ajax({
            url: url,
            type: 'delete',
            headers: {
                userUuid: userUuid,
                "Content-Type": "application/json"
            },
            success: function () {
                var auth2 = gapi.auth2.getAuthInstance()
                if (auth2 !== null) {
                    auth2.signOut().then(function () {
                        onSignedOut()
                    })
                } else {
                    onSignedOut()
                }
            },
            error: function (xhr, ajaxOptions, thrownError) {
                console.log(xhr.status);
                console.log(thrownError);
            }
        });
    }

    function requestAllowNewsletter(allow, onFinished) {
        var allowNewsletters = {}
        allowNewsletters[newsletter_name] = allow
        $.ajax({
            url: userEndpoint + "/me",
            type: 'patch',
            data: JSON.stringify({
                allowNewsletters: allowNewsletters
            }),
            headers: {
                userUuid: userUuid,
                "Content-Type": "application/json"
            },
            success: function (data) {
                onFinished()
            },
            error: function (xhr, ajaxOptions, thrownError) {
                console.log(xhr.status)
                console.log(thrownError)
                onFinished()
            }
        });
    }

    function signOut(onSignedOut) {
        requestDeleteUserAndSignOutGoogle(onSignedOut, userEndpoint + "/logout")
    }

    function removeAccount(onSignedOut) {
        requestDeleteUserAndSignOutGoogle(onSignedOut, userEndpoint)
    }

    function sendFeedback(comment, onSent) {
        $.ajax({
            url: feedbackEndpoint,
            type: 'post',
            data: JSON.stringify({
                type: "PRIVATE",
                pageKey: pageKey,
                comment: comment
            }),
            headers: {
                userUuid: userUuid,
                "Content-Type": "application/json"
            },
            success: function (userData) {
                onSent(userData)
            },
            error: function (xhr, ajaxOptions, thrownError) {
                console.log(xhr.status);
                console.log(thrownError);
            }
        });
    }

    // *** Popups ***

    function showAllowNewsletterPopup(onAllowed, onRejected) {
        Swal.fire({
            title: 'Newsletter',
            html:
                'Would you like to receive updates from ' + name + ' on your email?',
            showCloseButton: true,
            showCancelButton: true,
            focusConfirm: false,
            allowOutsideClick: false,
            imageUrl: 'https://learningdriven.fun/assets/images/cat_paws.gif',
            imageWidth: 400,
            imageHeight: 300,
            imageAlt: 'Custom image',
            confirmButtonText:
                '<i class="fa fa-thumbs-up"></i> OK',
            confirmButtonAriaLabel: 'Thumbs up, great!',
            cancelButtonText:
                '<i class="fa fa-thumbs-down"></i>',
            cancelButtonAriaLabel: 'Thumbs down'
        }).then((result) => {
            if (result.value) {
                onAllowed()
            } else {
                onRejected()
            }
        })
    }

    function showRemoveAccountPopup(onRemoveAccount, onCancelled) {
        Swal.fire({
            title: 'Are you sure?',
            text: "Do you want to remove your account? You won't be able to revert this! All your comments will become anonymous.",
            icon: 'warning',
            showCancelButton: true,
            confirmButtonColor: '#3085d6',
            cancelButtonColor: '#d33',
            confirmButtonText: 'Yes, delete it!'
        }).then((result) => {
            if (result.value) {
                onRemoveAccount()
            } else {
                onCancelled()
            }
        })
    }

    function showRemovedPopup() {
        Swal.fire(
            'Deleted!',
            'Your file has been deleted.',
            'success'
        )
    }

    function promptComment(type) {
        Swal.fire({
            input: 'textarea',
            inputPlaceholder: 'Type your ' + type + ' here...',
            inputAttributes: {
                'aria-label': 'Type your comment here'
            },
            showCancelButton: true
        }).then((text) => {
            if (text.value) {
                sendFeedback(text.value, function () {
                    Swal.fire("Thank you for your feedback")
                })
            }
        })
    }

    // *** FAB ***

    function cleanContainer() {
        $("#container").empty();
    }

    function showSignInButton(onSignedIn) {
        $(`
            <button class="FAB_menu_button" id="fab_login" style="background-image: url(&quot;` + logo_url + `&quot;);"></button>
            `).appendTo($("#container"))
        gapi.load('auth2', function () {
            auth2 = gapi.auth2.init({
                client_id: '835056603764-mg5hkv6l94tbfubod8mjgjvaing4makt.apps.googleusercontent.com',
                cookiepolicy: 'none',
                // Request scopes in addition to 'profile' and 'email'
                //scope: 'additional_scope'
            });
            auth2.attachClickHandler(document.getElementById('fab_login'), {},
                function (googleUser) {
                    registerUser(googleUser, function (userData) {
                        if (typeof onSignedEvent === "function") onSignedEvent(userData)
                        loggedUserData = userData
                        cleanContainer()
                        onSignedIn(userData)
                    })
                }, function (error) {
                    console.log(error);
                });
        });
    }

    function showClosedFabMenu(userData) {
        let fabMenu = $(`
            <button class="FAB_menu_button" id="fab_open" style="background-image: url(&quot;` + logo_url + `&quot;);"></button>
            `);
        fabMenu.appendTo($("#container"));
        fabMenu.click(function () {
            cleanContainer()
            showOpenedFabMenu(userData)
        })
    }

    function hasNewsletter(userData) {
        return userData.newsletters.includes(newsletter_name)
    }

    function addNewsletter(userData) {
        if (!userData.newsletters) {
            userData.newsletters = []
        }
        userData.newsletters.push(newsletter_name)
    }

    // linkConfigs: {text: string, link: string, icon: string}[]
    function buildExtraMenuLinks(linkConfigs) {
        let text = ""
        for (let i = 0; i < linkConfigs.length; i++) {
            const linkConfig = linkConfigs[i]
            text += `
                <li class="FAB_menu_account_navlink">
                    <a role="button" tabindex="0"
                       href="${linkConfig.link}"
                       rel="noopener noreferrer">
                        <div class="FAB_menu_account_navlink_icon">
                            <i class="${linkConfig.icon}"></i>
                        </div>
                        <span class="FAB_menu_account_navlink_text">${linkConfig.text}</span>
                    </a>
                </li>
            `
        }
        return text
    }

    function showOpenedFabMenu(userData) {
        var newsletterChecked = ""
        if (hasNewsletter(userData)) {
            newsletterChecked = "checked"
        }
        let fabMenu = $(`
            <div class="FAB_menu">
                <div class="FAB_menu_withoutHeader">
                    <div class="FAB_menu_account">
                        <div>
                            <h2 class="FAB_menu_account_title">My Account</h2>
                            <small class="FAB_menu_account_small">` + userData.email + `</small>
                        </div>
                        <div role="button" class="FAB_menu_account_img_button">
                            <img src="` + userData.imageUrl + `"
                                 class="FAB_menu_account_img">
                        </div>
                    </div>

                    <ul>
                        ${typeof extraMenuLinksFunction === "function" ? buildExtraMenuLinks(extraMenuLinksFunction(userData)) : ""}
    <!--                    <li class="FAB_menu_account_navlink">-->
    <!--                        <a role="button" tabindex="0">-->
    <!--                            <div class="FAB_menu_account_navlink_icon">-->
    <!--                                <i class="fas fa-user-edit"></i>-->
    <!--                            </div>-->
    <!--                            <span class="FAB_menu_account_navlink_text">Edit Profile</span>-->
    <!--                        </a>-->
    <!--                    </li>-->

    <!--                   <li class="FAB_menu_account_navlink">-->
    <!--                       <a role="button" tabindex="0" href="javascript:void(0)" id="comment">-->
    <!--                           <div class="FAB_menu_account_navlink_icon">-->
    <!--                               <i class="fas fa-comment"></i>-->
    <!--                           </div>-->
    <!--                           <span class="FAB_menu_account_navlink_text">Comment</span>-->
    <!--                       </a>-->
    <!--                   </li>-->
                        <li class="FAB_menu_account_navlink">
                            <a role="button" tabindex="0" href="javascript:void(0)" id="feedback">
                                <div class="FAB_menu_account_navlink_icon">
                                    <i class="fas fa-paper-plane"></i>
                                </div>
                                <span class="FAB_menu_account_navlink_text">Send private feedback</span>
                            </a>
                        </li>

                        <li class="FAB_menu_account_navlink">
                            <label class="switch" style="margin-right: 10px" id="LD-newsletter">
                                <input type="checkbox"  id="LD-newsletter-input" ` + newsletterChecked + `>
                                <span class="slider round"></span>
                            </label>
                            <a role="button" tabindex="0">
                                <span class="FAB_menu_account_navlink_text">Newsletter</span>
                            </a>
                        </li>
                        <li class="FAB_menu_account_navlink">
                            <a role="button" tabindex="0" href="javascript:void(0)" id="remove-account">
                                <div class="FAB_menu_account_navlink_icon">
                                    <i class="fas fa-trash"></i>
                                </div>
                                <span class="FAB_menu_account_navlink_text">Remove account</span>
                            </a>
                        </li>
                        <li class="FAB_menu_account_navlink">
                            <a role="button" tabindex="0" href="javascript:void(0)" id="sign-out">
                                <div class="FAB_menu_account_navlink_icon">
                                    <i class="fas fa-sign-out-alt"></i>
                                </div>
                                <span class="FAB_menu_account_navlink_text">Log Out</span>
                            </a>
                        </li>
                    </ul>
                </div>
            </div>
            <button class="FAB_menu_button" id="close_fab" style="transform: rotate(45deg);">
                <svg title="Close"
                     icon="M16.904 10H9.999v6.904a.997.997 0 1 1-1.994 0V9.999H1.101a.997.997 0 1 1 0-1.994h6.904V1.101a.997.997 0 1 1 1.994 0v6.904h6.905a.997.997 0 1 1 0 1.994z"
                     color="#444444" width="18px" cursor="pointer" display="block" class="clickable"
                     height="18px"
                     viewBox="0 0 18 18" role="img" aria-labelledby="svg-title"
                     style="cursor: pointer; display: block; vertical-align: middle;"><title id="svg-title">
                    Close</title>
                    <path d="M16.904 10H9.999v6.904a.997.997 0 1 1-1.994 0V9.999H1.101a.997.997 0 1 1 0-1.994h6.904V1.101a.997.997 0 1 1 1.994 0v6.904h6.905a.997.997 0 1 1 0 1.994z"
                          fill="#444444"></path>
                </svg>
            </button>
            `);
        fabMenu.appendTo($("#container"));

        function closeFab() {
            cleanContainer()
            showClosedFabMenu(userData)
        }

        $('#close_fab').click(function () {
            closeFab()
        })
        $(document).on('change', '#LD-newsletter-input', function () {
            requestAllowNewsletter(this.checked, function () { // onFinished
                // no-op
            })
        });
        //     $('#comment').click(function () {
        //         promptComment('comment')
        //         closeFab()
        //     })
        $('#feedback').click(function () {
            promptComment('private feedback')
            closeFab()
        })
        $("#remove-account").click(function () {
            showRemoveAccountPopup(function () { // on remove
                removeAccount(function () {
                    loggedUserData = undefined
                    if (typeof onSignedOutEvent === "function") onSignedOutEvent()
                    cleanContainer()
                    showLoginButton()
                })
            }, function () { // on cancelled
                // no-op
            })
        })
        $("#sign-out").click(function () {
            signOut(function () {
                loggedUserData = undefined
                if (typeof onSignedOutEvent === "function") onSignedOutEvent()
                cleanContainer()
                showLoginButton()
            })
        })
    }

    // *** Logic ***

    function showLoginButton() {
        showSignInButton(function (userData) {
            if (hasNewsletter(userData)) {
                showOpenedFabMenu(userData)
            } else {
                showAllowNewsletterPopup(function () {
                    requestAllowNewsletter(true, function () {
                        addNewsletter(userData);
                        showOpenedFabMenu(userData)
                    })
                }, function () {
                    showOpenedFabMenu(userData)
                })
            }
        })
    }

    fabVisible = false
    $(`
<div id="FAB_menu_container">
    <div style="overflow-y: auto; position: relative;">
        <div id="container">
        </div>
    </div>
</div>`).appendTo(document.body)

    getUserData(function (userData) {
        if (userData != null) {
            loggedUserData = userData
            showClosedFabMenu(userData)
            onSignedEvent(userData)
        } else {
            showLoginButton()
        }
        fabVisible = true
    })
}
